/********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE MP3FRAUNHOFERENC SOURCE CODE.           *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE MP3FRAUNHOFERENC SOURCE CODE IS (C) COPYRIGHT 2008 Cocha     *
 * http://sourceforge.jp/projects/directshow/                       *
 *                                                                  *
 ********************************************************************/

#include "MP3FraunhoferEnc.h"

static CAcmFraunhofer *g_pMyClass = NULL;

// -----------------------------------------------------------------------------------------------------------------------------------
CAcmFraunhofer::CAcmFraunhofer()
{  // RXgN^

   g_pMyClass = this;
   m_hDriver = NULL;
   m_hStream = NULL;
   m_pInBuffer = NULL;
   m_pOutBuffer = NULL;
}
// -----------------------------------------------------------------------------------------------------------------------------------
CAcmFraunhofer::~CAcmFraunhofer()
{  // fXgN^

   if(m_hDriver != NULL)
   {
      ::acmDriverClose(m_hDriver, 0);
      m_hDriver = NULL;
   }

   if(m_hStream != NULL)
   {
      ::acmStreamUnprepareHeader(m_hStream, &m_acmHeader, 0);
      ::acmStreamClose(m_hStream, 0);
      m_hStream = NULL;
   }

   if(m_pInBuffer != NULL)
   {
      delete[] m_pInBuffer;
      m_pInBuffer = NULL;
   }

   if(m_pOutBuffer != NULL)
   {
      delete[] m_pOutBuffer;
      m_pOutBuffer = NULL;
   }
}
// -----------------------------------------------------------------------------------------------------------------------------------
HRESULT CAcmFraunhofer::CheckFormat(WAVEFORMATEX *pwf, int nOutBitrate)
{  // pwf̓͂̏ꍇɁAnOutBitratẽrbg[gŏo͂ł邩`FbN

   // 񋓂ŕKvȃf[^
   ::CopyMemory(&m_enumData.inFormat, pwf, sizeof(m_enumData.inFormat));
   m_enumData.nBitrate = nOutBitrate;
   ::ZeroMemory(&m_enumData.outFormat, sizeof(m_enumData.outFormat));

   // R[fbN񋓂āAڕW̃tH[}bg擾
   ::acmDriverEnum(AcmDriverEnumCallback, 0, 0);

   // ڕW̃tH[}bg擾łĂȂȂfalseԂ
   if(m_enumData.outFormat.wfx.nSamplesPerSec == 0)
      return S_FALSE;

  return S_OK;
}
// -----------------------------------------------------------------------------------------------------------------------------------
BOOL CALLBACK CAcmFraunhofer::AcmDriverEnumCallback(HACMDRIVERID hadid, DWORD dwInstance, DWORD fdwSupport)
{  // ACM

   if(fdwSupport & ACMDRIVERDETAILS_SUPPORTF_CODEC)
   {
      MMRESULT mmr;
      ACMDRIVERDETAILS details;
      HACMDRIVER driver;
      int i;
      details.cbStruct = sizeof(ACMDRIVERDETAILS);
      mmr = acmDriverDetails(hadid, &details, 0);
      mmr = acmDriverOpen(&driver, hadid, 0);

      // Fraunhofer̃R[fbN̓o[WɂẮA
      // szShortName̖ɃXy[X肷̂
      // szShortName̐擪12ŕr
      WCHAR awCodecName[32];

      for(i=0;i<12;i++)
         awCodecName[i] = details.szShortName[i];

      awCodecName[i] = TEXT('\0');

      if(::lstrcmp(awCodecName, TEXT("MPEG Layer-3")) == 0)
      {
         for(i=0;i<(int)details.cFormatTags;i++)
         {
            ACMFORMATTAGDETAILS fmtDetails;
            ZeroMemory(&fmtDetails, sizeof(fmtDetails));

            fmtDetails.cbStruct = sizeof(ACMFORMATTAGDETAILS);
            fmtDetails.dwFormatTagIndex = i;
            mmr = acmFormatTagDetails(driver,&fmtDetails, ACM_FORMATTAGDETAILSF_INDEX);

            if(fmtDetails.dwFormatTag == WAVE_FORMAT_MPEGLAYER3)
            {
               for(int n=0;n<(int)fmtDetails.cStandardFormats;n++)
               {  // rbg[g

                  MPEGLAYER3WAVEFORMAT mwf;

                  ACMFORMATDETAILS formatDetails;
                  formatDetails.pwfx          = (WAVEFORMATEX *)&mwf;
                  formatDetails.cbStruct      = sizeof(ACMFORMATDETAILS);
                  formatDetails.fdwSupport    = 0;
                  formatDetails.dwFormatTag   = WAVE_FORMAT_MPEGLAYER3;
                  formatDetails.dwFormatIndex = n;
		            acmMetrics(NULL, ACM_METRIC_MAX_SIZE_FORMAT, &formatDetails.cbwfx); 

                  acmFormatDetails(driver, &formatDetails, ACM_FORMATDETAILSF_INDEX);

                  if((mwf.wfx.nAvgBytesPerSec * 8) == g_pMyClass->m_enumData.nBitrate &&
                      mwf.wfx.nSamplesPerSec == g_pMyClass->m_enumData.inFormat.nSamplesPerSec &&
                      mwf.wfx.nChannels == g_pMyClass->m_enumData.inFormat.nChannels)
                  {
                     // ()ۂɕϊł邩`FbN
                     mmr = ::acmStreamOpen(NULL, driver, &g_pMyClass->m_enumData.inFormat, (WAVEFORMATEX *)&mwf, NULL, 0, 0, ACM_STREAMOPENF_QUERY);
                     if(mmr == 0)
                     {  // Kvȃf[^i[āA񋓏𔲂
                        g_pMyClass->m_enumData.hadid = hadid;
                        ::CopyMemory(&g_pMyClass->m_enumData.outFormat, &mwf, sizeof(mwf));
                        mmr = ::acmDriverClose(driver, 0);
                        return FALSE;
                     }
                  }
               }
            }
         }
      }

      mmr = ::acmDriverClose(driver, 0);
   }

   return TRUE;
}
// ---------------------------------------------------------------------------------------------------------------------------------------------------------
HRESULT CAcmFraunhofer::GetOutFormat(WAVEFORMATEX *pInFormat, int nBitrate, MPEGLAYER3WAVEFORMAT *pOutFormat)
{  // o͂̃tH[}bg擾

   MMRESULT rc;
   int nOutBufferSize = -1;

   HACMDRIVER hDriver = NULL;
   HACMSTREAM hStream = NULL;

   // 񋓂ŕKvȃf[^
   ::CopyMemory(&m_enumData.inFormat, pInFormat, sizeof(m_enumData.inFormat));
   m_enumData.nBitrate = nBitrate;
   ::ZeroMemory(&m_enumData.outFormat, sizeof(m_enumData.outFormat));

   // R[fbN񋓂āAڕW̃tH[}bg擾
   ::acmDriverEnum(AcmDriverEnumCallback, 0, 0);
   if(m_enumData.outFormat.wfx.nSamplesPerSec == 0)
      goto end;

   // ACMopen
   rc = ::acmDriverOpen(&hDriver, m_enumData.hadid, 0);
   if(rc != 0) goto end;

   rc = ::acmStreamOpen(&hStream, hDriver, pInFormat, (WAVEFORMATEX *)&m_enumData.outFormat, NULL, 0, 0, ACM_STREAMOPENF_NONREALTIME);
   if(rc != 0) goto end;

   // o͂̃tH[}bgi[
   ::CopyMemory(pOutFormat, &m_enumData.outFormat, sizeof(m_enumData.outFormat));

   if(m_enumData.outFormat.wfx.nSamplesPerSec == 0)
      goto end;

end:

   if(hDriver != NULL)
   {
      ::acmDriverClose(hDriver, 0);
      hDriver = NULL;
   }

   if(hStream != NULL)
   {
      ::acmStreamClose(hStream, 0);
      hStream = NULL;
   }

   return S_OK;
}
// ---------------------------------------------------------------------------------------------------------------------------------------------------------
int CAcmFraunhofer::GetOutBufferSize(WAVEFORMATEX *pInFormat, int nInBufferSize, int nBitrate)
{  // o͂̃obt@TCY擾

   MMRESULT rc;
   int nOutBufferSize = -1;

   HACMDRIVER hDriver = NULL;
   HACMSTREAM hStream = NULL;

   // 񋓂ŕKvȃf[^
   ::CopyMemory(&m_enumData.inFormat, pInFormat, sizeof(m_enumData.inFormat));
   m_enumData.nBitrate = nBitrate;
   ::ZeroMemory(&m_enumData.outFormat, sizeof(m_enumData.outFormat));

   // R[fbN񋓂āAڕW̃tH[}bg擾
   ::acmDriverEnum(AcmDriverEnumCallback, 0, 0);
   if(m_enumData.outFormat.wfx.nSamplesPerSec == 0)
      goto end;

   // ACMopen
   rc = ::acmDriverOpen(&hDriver, m_enumData.hadid, 0);
   if(rc != 0) goto end;

   rc = ::acmStreamOpen(&hStream, hDriver, pInFormat, (WAVEFORMATEX *)&m_enumData.outFormat, NULL, 0, 0, ACM_STREAMOPENF_NONREALTIME);
   if(rc != 0) goto end;

   // o͂̃f[^TCYZo
   DWORD dwOutSize;
   rc = acmStreamSize(hStream, nInBufferSize, &dwOutSize, ACM_STREAMSIZEF_SOURCE);
   if(rc != 0) goto end;

   nOutBufferSize = dwOutSize;

end:

   if(hDriver != NULL)
   {
      ::acmDriverClose(hDriver, 0);
      hDriver = NULL;
   }

   if(hStream != NULL)
   {
      ::acmStreamClose(hStream, 0);
      hStream = NULL;
   }

   return nOutBufferSize;
}
// ---------------------------------------------------------------------------------------------------------------------------------------------------------
HRESULT CAcmFraunhofer::ConvertInit(WAVEFORMATEX *pInFormat, int nInBufferSize, MPEGLAYER3WAVEFORMAT *pOutFormat, int *pOutBufferSize, int nBitrate)
{
   MMRESULT rc;
   DWORD dwOutSize;

   // ϐ
   if(m_hDriver != NULL)
   {
      ::acmDriverClose(m_hDriver, 0);
      m_hDriver = NULL;
   }

   if(m_hStream != NULL)
   {
      ::acmStreamUnprepareHeader(m_hStream, &m_acmHeader, 0);
      ::acmStreamClose(m_hStream, 0);
      m_hStream = NULL;
   }

   if(m_pInBuffer != NULL)
   {
      delete[] m_pInBuffer;
      m_pInBuffer = NULL;
   }

   if(m_pOutBuffer != NULL)
   {
      delete[] m_pOutBuffer;
      m_pOutBuffer = NULL;
   }


   // 񋓂ŕKvȃf[^
   ::CopyMemory(&m_enumData.inFormat, pInFormat, sizeof(m_enumData.inFormat));
   m_enumData.nBitrate = nBitrate;
   ::ZeroMemory(&m_enumData.outFormat, sizeof(m_enumData.outFormat));

   // R[fbN񋓂āAڕW̃tH[}bg擾
   ::acmDriverEnum(AcmDriverEnumCallback, 0, 0);
   if(m_enumData.outFormat.wfx.nSamplesPerSec == 0)
      goto error;

   // ̓obt@쐬
   m_pInBuffer = new BYTE[nInBufferSize];

   // ACMopen
   rc = ::acmDriverOpen(&m_hDriver, m_enumData.hadid, 0);


   rc = ::acmStreamOpen(&m_hStream, m_hDriver, pInFormat, (WAVEFORMATEX *)&m_enumData.outFormat, NULL, 0, 0, ACM_STREAMOPENF_NONREALTIME);
   if(rc != 0) goto error;

   // o͂̃f[^TCYZo
   rc = acmStreamSize(m_hStream, nInBufferSize, &dwOutSize, ACM_STREAMSIZEF_SOURCE);
   if(rc != 0) goto error;

   // o̓obt@쐬
   m_pOutBuffer = new BYTE[dwOutSize];
   *pOutBufferSize = dwOutSize;

   // acmwb_̍쐬
   ::ZeroMemory(&m_acmHeader, sizeof(ACMSTREAMHEADER));
   m_acmHeader.cbStruct    = sizeof(ACMSTREAMHEADER);
   m_acmHeader.cbSrcLength = nInBufferSize;
   m_acmHeader.pbSrc       = m_pInBuffer;
   m_acmHeader.pbDst       = m_pOutBuffer;
   m_acmHeader.cbDstLength = dwOutSize;

   rc = acmStreamPrepareHeader(m_hStream, &m_acmHeader, 0);
   if(rc != 0) goto error;

   // o͂̃tH[}bgi[
   ::CopyMemory(pOutFormat, &m_enumData.outFormat, sizeof(m_enumData.outFormat));

   return S_OK;


error:

   if(m_hDriver != NULL)
   {
      ::acmDriverClose(m_hDriver, 0);
      m_hDriver = NULL;
   }

   if(m_hStream != NULL)
   {
      ::acmStreamUnprepareHeader(m_hStream, &m_acmHeader, 0);
      ::acmStreamClose(m_hStream, 0);
      m_hStream = NULL;
   }

   if(m_pInBuffer != NULL)
   {
      delete[] m_pInBuffer;
      m_pInBuffer = NULL;
   }

   if(m_pOutBuffer != NULL)
   {
      delete[] m_pOutBuffer;
      m_pOutBuffer = NULL;
   }

   return S_FALSE;
}
// ---------------------------------------------------------------------------------------------------------------------------------------------------------
HRESULT CAcmFraunhofer::Convert(BYTE *pbInData, int nInDataLength, BYTE *pbOutData, int *pnOutDataLength)
{
   MMRESULT rc;

   // ̓f[^̏
   ::CopyMemory(&m_pInBuffer[0], &pbInData[0], nInDataLength);
   m_acmHeader.cbSrcLength = nInDataLength;

   // ϊ
   rc = ::acmStreamConvert(m_hStream, &m_acmHeader, ACM_STREAMCONVERTF_BLOCKALIGN);
   if(rc != 0) return S_FALSE;

   // o̓f[^̏
   if(m_acmHeader.cbDstLengthUsed > 0)
      ::CopyMemory(pbOutData, m_pOutBuffer, m_acmHeader.cbDstLengthUsed);

   *pnOutDataLength = m_acmHeader.cbDstLengthUsed;

   return S_OK;
}
// ---------------------------------------------------------------------------------------------------------------------------------------------------------
HRESULT CAcmFraunhofer::ConvertEnd(BYTE *pbOutData, int *pnOutDataLength)
{
   MMRESULT rc;

   // ŌACMobt@ɗ܂Ăf[^̕ϊ
   m_acmHeader.cbSrcLength = 0;
   rc = ::acmStreamConvert(m_hStream, &m_acmHeader, ACM_STREAMCONVERTF_END);
   if(rc != 0) return S_FALSE;

   if(m_acmHeader.cbDstLengthUsed > 0)
      ::CopyMemory(pbOutData, m_pOutBuffer, m_acmHeader.cbDstLengthUsed);

   *pnOutDataLength = m_acmHeader.cbDstLengthUsed;

   return S_OK;
}
// ---------------------------------------------------------------------------------------------------------------------------------------------------------
